

A database field is a single piece of information from a record. A database record is a set of fields.

The properties of a field describe the characteristics and behavior of data added to that field.A field's data type is the most important property because it determines what kind of data the field can store.

Field Type Parameters - Tells additional details to the database about how to store a value and configures the general behavior of the input boxes.

The field can be used in five different ways :

1. {{ fieldname }} or {{ fieldname(params) }} - Returns processed value of the field using the field type parameters. If it's a float field type then the returned value will be 123.00 for example instead of 123.
2. {{ fieldname.value }} or {{ fieldname(params) }} - Returns pure, unprocessed value of the field.
3. {{ fieldname.title }} - Returns the field title in current language.
4. {{ fieldname.label }} or {{ fieldname.label(clickable = true|false) }} - Renders the label HTML tag with the field title in current language. It can be clickable to sort the records by that field.
5. {{ fieldname.edit }} or {{ fieldname(params) }} - Returns an edit record input box and the params configures the appearance and functionality.

Below is the list of parameters every field type accepts and how to use it :

# Alias (For SEO Links)

Alias field that can be used instead of listing_id for SEO purpose.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

# Article

Select Article box.

**Field Type Parameters:**

1. Category ID - Category ID to allow selection from that category only.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Author User

Stores the ID of the user who created the record.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Blob (Save file into the database)

A BLOB is a binary large object that can hold an entire file.

**Field Type Parameters:**

1. Maximum length - Maximum length of value to be held

    

* **tiny** - (255 Bytes (TINYBLOB). A binary large object column with a maximum length of 255 (2^8 - 1) bytes. Each TINYBLOB value is stored using a one-byte length prefix that indicates the number of bytes in the value.)

    

* (Default. 64KB (BLOB). A binary large object column with a maximum length of 65535 (2^16 - 1) bytes, or 64KB in storage. Each BLOB value is stored using a two-byte length prefix that indicates the number of bytes in the value.)

    

* **medium** - (16MB (MEDIUMBLOB). A binary large object column with a maximum length of 16777215 (2^24 - 1) bytes, or 16MB in storage. Each MEDIUMBLOB value is stored using a three-byte length prefix that indicates the number of bytes in the value.)

    

* **long** - (4GB (LONGBLOB). A binary large object column with a maximum length of 4294967295 (2^32 - 1) bytes, or 4GB in storage. Each LONGBLOB value is stored using a four-byte length prefix that indicates the number of bytes in the value.)

2. File Types - List of acceptable file types (list separated by space)

3. Field to get the file name from / to - Specify the name of the field where to store the original file name. By the default file name will be the current timestamp and guessed file extension.

4. Custom PHP - PHP file/function that will process file content before being downloaded by the user. File must be located at /components/com_customtables/customphp.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Security - Encrypts the link to the file and limits access time to it.

    

* (Default. Public. Direct link to the file.)

    

* **timelimited** - (Time Limited (8-24 minutes))

    

* **timelimited_longterm** - (Time Limited (1.5 - 4 hours))

    

* **hostlimited** - (Time/Host Limited (8-24 minutes))

    

* **hostlimited_longterm** - (Time/Host Limited (1.5 - 4 hours))

    

* **private** - (Time/Host/User Limited (8-24 minutes))

    

* **private_longterm** - (Time/Host/User Limited (1.5 - 4 hours))

2. Link - How to render the link

    

* **icon-filename-link** - (Clickable Icon and File Name)

    

* **icon-link** - (Clickable Icon)

    

* **filename-link** - (Clickable File Name)

    

* **link-anchor** - (Clickable Link)

    

* **icon** - (Icon (IMG tag))

    

* (Default. Link Only)

    

* **filename** - (File Name)

    

* **link-to-icon** - (Link to Icon file)

    

* **extension** - (Extension)

    

* **file-size** - (File Size)

3. Icon - Select the icon size

    

* **16** - (16px X 16px Icon)

    

* **32** - (32px X 32px Icon)

    

* **48** - (48px X 48px Icon)

4. Target Window - Target browser window when opening the link

    

* (Default. Parent)

    

* **_blank** - (New Window With Navigation)

    

* **savefile** - (Open Save File As Dialog Window)

`{{ FieldName("timelimited","filename-link",32,"parent") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Change Log

Logs all record modifications. User, Datetime, IP. You can return to the previous version of the record.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Change Time

Stores the date/time when record was last modified.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Format - PHP date format i.e. 'Y-m-d' or Joomla defined format (language supported) if left empty. 'timestamp' to return the datetime in UNIX timestamp format.

`{{ FieldName("Y-m-d") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Checkbox (YES/NO)

Provides a single checkbox to be checked or unchecked

![Checkbox (YES/NO)](https://joomlaboat.com/images/components/ct/field-types/checkbox.png)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

3. Format - Type of checkbox

    

* (Default. Simple HTML Checkbox)

    

* **yesno** - (Yes/No buttons)

`{{ FieldName.edit("yesno") }}`
# Color

Provides Color Picker functionality in the Edit Form or hexadecimal value as an output.

![Color](https://joomlaboat.com/images/components/ct/field-types/color.png)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Format - Return formats: HEX is #rrggbb or #rrggbbaa, RGB is rgb(r,g,b) or rgbs(r,g,b,a)

    

* (Hexadecimal - default)

    

* **rgba** - (rgba())

`{{ FieldName("rgba") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. Transparent - Allow transparency - alpha color.

    

* (No - default)

    

* **transparent** - (Yes)

`{{ FieldName.edit("transparent") }}`
# Creation Time

Stores the date/time when record was created.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Format - PHP date format i.e. 'Y-m-d' or Joomla defined format (language supported) if left empty. 'timestamp' to return the datetime in UNIX timestamp format.

`{{ FieldName("Y-m-d") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Custom Record ID

Autoincrement ID. parameter: start from number

**Field Type Parameters:**

1. Start Number - The value starts from number

    

* **Default** - 1

    

* **Min** - 1

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

# Date

Date ot Date/Time Picker

**Field Type Parameters:**

1. Type - Date or Date / Time

    

* **date** - (Date Only. A date, supported range is 1000-01-01 to 9999-12-31)

    

* **datetime** - (Date and Time. A date and time combination, supported range is 1000-01-01 00:00:00 to 9999-12-31 23:59:59)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Format - PHP date format i.e. 'Y-m-d H:i:s' or Joomla defined format (language supported) if left empty. 'timestamp' to return the datetime in UNIX timestamp format.

`{{ FieldName("Y-m-d") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

3. Format - PHP date format i.e. 'Y-m-d H:i:s'

`{{ FieldName.edit("Y-m-d") }}`
# Email

Provides an email field.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# File

Provides File Upload functionality.

**Field Type Parameters:**

1. Max File Size - Obsolete. The max. upload file size is taken from PHP.ini file. No need to specify anything.

2. Folder - Folder where all the files will be kept. By default /images

3. File Types - List of acceptable file types (list separated by space)

4. Save As File Name - Specify the name of a newly uploaded file. You may use field tags [fieldname]

5. Custom PHP - PHP file/function that will process file content before being downloaded by the user. File must be located at /components/com_customtables/customphp.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Security - Encrypts the link to the file and limits access time to it.

    

* (Default. Public. Direct link to the file.)

    

* **timelimited** - (Time Limited (8-24 minutes))

    

* **timelimited_longterm** - (Time Limited (1.5 - 4 hours))

    

* **hostlimited** - (Time/Host Limited (8-24 minutes))

    

* **hostlimited_longterm** - (Time/Host Limited (1.5 - 4 hours))

    

* **private** - (Time/Host/User Limited (8-24 minutes))

    

* **private_longterm** - (Time/Host/User Limited (1.5 - 4 hours))

2. Link - How to render the link

    

* **icon-filename-link** - (Clickable Icon and File Name)

    

* **icon-link** - (Clickable Icon)

    

* **filename-link** - (Clickable File Name)

    

* **link-anchor** - (Clickable Link)

    

* **icon** - (Icon (IMG tag))

    

* (Default. Link Only)

    

* **filename** - (File Name)

    

* **link-to-icon** - (Link to Icon file)

    

* **extension** - (Extension)

3. Icon - Select the icon size

    

* **16** - (16px X 16px Icon)

    

* **32** - (32px X 32px Icon)

    

* **48** - (48px X 48px Icon)

4. Target Window - Target browser window when opening the link

    

* (Default. Parent)

    

* **_blank** - (New Window With Navigation)

    

* **savefile** - (Open Save File As Dialog Window)

`{{ FieldName("timelimited","filename-link",32,"parent") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# File Box

Provides Multiple File Upload functionality. It also tracks versions.

**Field Type Parameters:**

1. Max File Size - Maximum file size that can be uploaded to the server (in megabytes). Note: check your PHP settings as well.

2. Folder - Folder where all the files will be kept. By default /images

3. File Type - List of acceptable file types (list separated by space)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Security - Encrypts the link to the file and limits access time to it.

    

* (Default. Public. Direct link to the file.)

    

* **timelimited** - (Time Limited (8-24 minutes))

    

* **timelimited_longterm** - (Time Limited (1.5 - 4 hours))

    

* **hostlimited** - (Time/Host Limited (8-24 minutes))

    

* **hostlimited_longterm** - (Time/Host Limited (1.5 - 4 hours))

    

* **private** - (Time/Host/User Limited (8-24 minutes))

    

* **private_longterm** - (Time/Host/User Limited (1.5 - 4 hours))

2. Link - How to render the link

    

* **icon-filename-link** - (Clickable Icon and File Name)

    

* **icon-link** - (Clickable Icon)

    

* **filename-link** - (Clickable File Name)

    

* **link-anchor** - (Clickable Link)

    

* **icon** - (Icon (IMG tag))

    

* (Default. Link Only)

    

* **filename** - (File Name)

    

* **link-to-icon** - (Link to Icon file)

    

* **extension** - (Extension)

    

* **file-size** - (File Size)

3. Icon - Select the icon size

    

* **16** - (16px X 16px Icon)

    

* **32** - (32px X 32px Icon)

    

* **48** - (48px X 48px Icon)

4. Target Window - Target browser window when opening the link.

    

* (Default. Parent)

    

* **_blank** - (New Window With Navigation)

    

* **savefile** - (Open Save File As Dialog Window)

5. List Format - How to the output list format.

    

* (Default. Ordered List - OL and OL HTML tags)

    

* **ul** - (Unordered List - UL and LI HTML tags)

    

* **,** - (Comma separated list)

    

* **;** - (The list separated by semicolon)

`{{ FieldName("timelimited","filename-link",32,"parent",",") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# File Link

Path to a folder on the server. Provides file selection from particular folder.

**Field Type Parameters:**

1. Folder - Folder a user can select a file from.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Security - Encrypts the link to the file and limits access time to it.

    

* (Default. Public. Direct link to the file.)

    

* **timelimited** - (Time Limited (8-24 minutes))

    

* **timelimited_longterm** - (Time Limited (1.5 - 4 hours))

    

* **hostlimited** - (Time/Host Limited (8-24 minutes))

    

* **hostlimited_longterm** - (Time/Host Limited (1.5 - 4 hours))

    

* **private** - (Time/Host/User Limited (8-24 minutes))

    

* **private_longterm** - (Time/Host/User Limited (1.5 - 4 hours))

2. Link - How to render the link

    

* **icon-filename-link** - (Clickable Icon and File Name)

    

* **icon-link** - (Clickable Icon)

    

* **filename-link** - (Clickable File Name)

    

* **link-anchor** - (Clickable Link)

    

* **icon** - (Icon (IMG tag))

    

* (Default. Link Only)

    

* **filename** - (File Name)

    

* **link-to-icon** - (Link to Icon file)

    

* **extension** - (Extension)

3. Icon - Select the icon size

    

* **16** - (16px X 16px Icon)

    

* **32** - (32px X 32px Icon)

    

* **48** - (48px X 48px Icon)

4. Target Window - Target browser window when opening the link

    

* (Default. Parent)

    

* **_blank** - (New Window With Navigation)

    

* **savefile** - (Open Save File As Dialog Window)

`{{ FieldName("timelimited","filename-link",32,"parent") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Float (Decimal) Number

Provides a decimal number inputbox.

**Field Type Parameters:**

1. Decimals - Number of digits to the right of the decimal point (the scale). Must be no larger than Precision Number of digits.

    

* **Default** - 2

    

* **Min** - 1

    

* **Max** - 30

2. Precision - Maximum number of digits (the precision).

    

* **Default** - 20

    

* **Min** - 1

    

* **Max** - 65

3. Smart Input - Allows input without pressing decimal point key. Example: to type [12.67] just type [1267] The decimal point will be placed by itself.

    

* (No - default)

    

* **smart** - (Yes)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Number of decimals - Format returning number. Add tracing 0 if needed. (useful when formatting a currency)

    

* **Default** -

    

* **Max** - 30

2. Decimal Separator - Separate decimals with provided character. i.e. ',' or '.'

3. Thousand Separator - Separate thousands with provided character. i.e. ',' or '.' or ' ' (space)

`{{ FieldName(0,".",",") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

`{{ FieldName.edit("inputboxClass","readonly='readonly'") }}`
# GPS Coordinates

Google Map Coordinates (Latitude and Longitude) with selection map

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Format - Format the coordinates output or render a map.

    

* (Default. Latitude and Longitude separated by comma)

    

* **map** - (Google Map)

    

* **latitude** - (Latitude only)

    

* **longitude** - (Longitude only)

2. Map Width - Map Width in Pixels or %

3. Map Height - Map Width in Pixels or %

4. Map Zoom - The Google Maps API provides map tiles at various zoom levels for map type imagery. Most roadmap imagery is available from zoom levels 0 to 18

    

* **0** - (0)

    

* **1** - (1)

    

* **2** - (2)

    

* **3** - (3)

    

* **4** - (4)

    

* **5** - (5)

    

* **6** - (6)

    

* **7** - (7)

    

* **8** - (8)

    

* **9** - (9)

    

* **10** - (10)

    

* **11** - (11)

    

* **12** - (12)

    

* **13** - (13)

    

* **14** - (14)

    

* **15** - (15)

    

* **16** - (16)

    

* **17** - (17)

    

* **18** - (18)

`{{ FieldName("map",0) }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Image File

Provides Image Upload functionality.

**Field Type Parameters:**

1. Image Size Options - You may specify various size options. When image file being uploaded, the Custom Tables will create a modified version(s) of the image file based on the settings provided.

2. Image Processing Methods - Image processing methods that will be applied to newly uploaded image. This option available in Pro Version Only.

3. Folder - Folder where all the files will be kept. By default /images

4. Save As File Name

    

* (Default. Date Time (YmdHis))

    

* **transliterated** - (Filename (Transliterated). Alphanumeric and transliterated also the whitespace is replaced with the underscore character. Extra Number can be added if the file name is not unique.)

    

* **original** - (Filename As It Is. File name as it is, no changes. Extra Number can be added if the file name is not unique.)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Image Gallery

Provides Multiple Image Upload functionality.

**Field Type Parameters:**

1. Image Size Options - You may specify various size options. When image file being uploaded, the Custom Tables will create a modified version(s) of the image file based on the settings provided.

2. Image Processing Methods - Image processing methods that will be applied to a newly uploaded image. This option available in Pro Version Only.

3. Folder - Folder where all the files will be kept. By default /images

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Size - Select the image size variant.

2. List Separator - Separate images with a character. i.e. ',' or ';'

`{{ FieldName("preview",";") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Integer Number

Provides an integer number inputbox.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Language

Provides Language Selection box.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# MD5 Hash

MD5 Hash calculated based on selected fields values.

**Field Type Parameters:**

1. List of Fields - Field Values will be taken and MD5 Hash code will be calculated over it.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

# Multilingual Text String (Single Line)

Provides a set of text boxes for data entry. One inputbox for each language.

**Field Type Parameters:**

1. Length - Number of characters. Number of characters not bytes.

    

* **Default** - 255

    

* **Min** - 1

    

* **Max** - 1024

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Limit Output - Limits returning output text, first number of characters or first number of words.

    

* (Default. Returns as it is)

    

* **chars** - (Returns first number of characters from text that was trimmed, sanitized and HTML tags stripped.)

    

* **words** - (Returns first number of words from text that was trimmed, sanitized and HTML tags stripped.)

    

* **firstimage** - (Returns the link to a first image found in the text.)

2. Length - Number of characters or words.

    

* **Default** -

    

* **Min** - 1

    

* **Max** - 1024

3. Clean Braces - Deletes all braces from the text. (i.e. to avoid Content Plugins running.)

    

* (No - default)

    

* **true** - (Yes)

4. Clean Quotes - Deletes all quotes from the text.

    

* (No - default)

    

* **true** - (Yes)

5. Language - Specify the language. Default front-end language will be used if not specified.

`{{ FieldName("chars",1,true,true) }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Multilingual Text Area (Multiple Lines)

Provides a set of textarea boxes for data entry. One textarea box for each language.

**Field Type Parameters:**

1. Text Editor

    

* (Default. No Editor)

    

* **rich** - (Rich Text Editor)

2. Spell Checker

    

* (No - default)

    

* **spellcheck** - (Yes)

3. Maximum length - Maximum length of value to be held. The effective maximum length is less. The Joomla default utf8mb4 Character Set (4-Byte UTF-8 Unicode Encoding).

    

* **tiny** - (Up to 255 Bytes - 63 characters (TINYTEXT). TINYTEXT can store up to 63 Unicode characters i.e 255 bytes. It is suitable for storing short comments, captions, titles etc.)

    

* (Default. Up to 64KB - 16,383 characters (TEXT). TEXT can store up to 16,383 Unicode characters i.e 64KB. It is suitable for storing summaries of articles, short notices, captions, etc.)

    

* **medium** - (Up to 16MB - 4.2 million characters (MEDIUMTEXT). MEDIUMTEXT can store up to 4,194,303 Unicode characters i.e 16MB. It is suitable for larger text strings like books, research papers, and code backup.)

    

* **long** - (Up to 4GB - 10.7 billion characters (LONGTEXT). LONGTEXT can store up to 1,073,741,822 Unicode characters i.e 4GB. This is more than enough storage for any long-form text strings. For example, a book that MEDIUMTEXT can’t hold can be stored using LONGTEXT.)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Limit Output - Limits returning output text, first number of characters or first number of words.

    

* (Default. Returns as it is)

    

* **chars** - (Returns first number of characters from text that was trimmed, sanitized and HTML tags stripped.)

    

* **words** - (Returns first number of words from text that was trimmed, sanitized and HTML tags stripped.)

    

* **firstimage** - (Returns the link to a first image found in the text.)

2. Length - Number of characters or words.

    

* **Default** -

    

* **Min** - 1

    

* **Max** - 1024

3. Clean Braces - Deletes all braces from the text. (i.e. to avoid Content Plugins running.)

    

* (No - default)

    

* **true** - (Yes)

4. Clean Quotes - Deletes all quotes from the text.

    

* (No - default)

    

* **true** - (Yes)

5. Language - Specify the language. Default front-end language will be used if not specified.

`{{ FieldName("chars",1,true,true) }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Ordering/Sorting

Record custom position, a front-end tool to sort records in a custom order.

![Ordering/Sorting](https://joomlaboat.com/images/components/ct/field-types/ordering-catalog.png)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

# Radio

Radio Buttons

**Field Type Parameters:**

1. Parameters

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Server Info

Server and execution environment variables. See: https://php.net/manual/en/reserved.variables.server.php

**Field Type Parameters:**

1. Option - Select what information to save.

    

* **REMOTE_ADDR** - (REMOTE_ADDR. The IP address from which the user is viewing the current page. )

    

* **HTTP_REFERER** - (HTTP_REFERER. The address of the page (if any) which referred the user agent to the current page. This is set by the user agent. Not all user agents will set this, and some provide the ability to modify HTTP_REFERER as a feature. In short, it cannot really be trusted.)

    

* **REMOTE_HOST** - (REMOTE_HOST. The Host name from which the user is viewing the current page. The reverse dns lookup is based off the REMOTE_ADDR of the user.)

    

* **REMOTE_PORT** - (REMOTE_PORT. The port being used on the user's machine to communicate with the web server. )

    

* **REMOTE_USER** - (REMOTE_USER. The authenticated user.)

    

* **PHP_SELF** - (PHP_SELF. The filename of the currently executing script, relative to the document root.)

    

* **GATEWAY_INTERFACE** - (GATEWAY_INTERFACE. Revision of the CGI specification the server is using; i.e. 'CGI/1.1')

    

* **SERVER_ADDR** - (SERVER_ADDR. The IP address of the server under which the current script is executing.)

    

* **SERVER_NAME** - (SERVER_NAME. The name of the server host under which the current script is executing. If the script is running on a virtual host, this will be the value defined for that virtual host.)

    

* **SERVER_SOFTWARE** - (SERVER_SOFTWARE. Server identification string, given in the headers when responding to requests.)

    

* **SERVER_PROTOCOL** - (SERVER_PROTOCOL. Name and revision of the information protocol via which the page was requested; i.e. 'HTTP/1.0')

    

* **REQUEST_METHOD** - (REQUEST_METHOD. Which request method was used to access the page; i.e. 'GET', 'HEAD', 'POST', 'PUT'.)

    

* **REQUEST_TIME** - (REQUEST_TIME. The timestamp of the start of the request. Available since PHP 5.1.0.)

    

* **REQUEST_TIME_FLOAT** - (REQUEST_TIME_FLOAT. The timestamp of the start of the request, with microsecond precision. Available since PHP 5.4.0.)

    

* **QUERY_STRING** - (QUERY_STRING. The query string, if any, via which the page was accessed. )

    

* **DOCUMENT_ROOT** - (DOCUMENT_ROOT. The document root directory under which the current script is executing, as defined in the server's configuration file.)

    

* **HTTP_ACCEPT** - (HTTP_ACCEPT. Contents of the Accept: header from the current request, if there is one.)

    

* **HTTP_ACCEPT_CHARSET** - (HTTP_ACCEPT_CHARSET. Contents of the Accept-Charset: header from the current request, if there is one. Example: 'iso-8859-1,*,utf-8'.)

    

* **HTTP_ACCEPT_ENCODING** - (HTTP_ACCEPT_ENCODING. Contents of the Accept-Encoding: header from the current request, if there is one. Example: 'gzip'.)

    

* **HTTP_ACCEPT_LANGUAGE** - (HTTP_ACCEPT_LANGUAGE. Contents of the Accept-Language: header from the current request, if there is one. Example: 'en'.)

    

* **HTTP_CONNECTION** - (HTTP_CONNECTION. Contents of the Connection: header from the current request, if there is one. Example: 'Keep-Alive'. )

    

* **HTTP_HOST** - (HTTP_HOST. Contents of the Host: header from the current request, if there is one. )

    

* **HTTP_USER_AGENT** - (HTTP_USER_AGENT. Contents of the User-Agent: header from the current request, if there is one. This is a string denoting the user agent which is accessing the page. A typical example is: Mozilla/4.5 [en] (X11; U; Linux 2.2.9 i586). Among other things, you can use this value with get_browser() to tailor your page's output to the capabilities of the user agent.)

    

* **HTTPS** - (HTTPS. Set to a non-empty value if the script was queried through the HTTPS protocol.)

    

* **REDIRECT_REMOTE_USER** - (REDIRECT_REMOTE_USER. The authenticated user if the request is internally redirected. )

    

* **SCRIPT_FILENAME** - (SCRIPT_FILENAME. The absolute pathname of the currently executing script.)

    

* **SERVER_ADMIN** - (SERVER_ADMIN. The value given to the SERVER_ADMIN (for Apache) directive in the web server configuration file. If the script is running on a virtual host, this will be the value defined for that virtual host. )

    

* **SERVER_PORT** - (SERVER_PORT. The port on the server machine being used by the web server for communication. For default setups, this will be '80'; using SSL, for instance, will change this to whatever your defined secure HTTP port is.)

    

* **SERVER_SIGNATURE** - (SERVER_SIGNATURE. String containing the server version and virtual host name which are added to server-generated pages, if enabled.)

    

* **PATH_TRANSLATED** - (PATH_TRANSLATED. Filesystem- (not document root-) based path to the current script, after the server has done any virtual-to-real mapping.)

    

* **SCRIPT_NAME** - (SCRIPT_NAME. Contains the current script's path. This is useful for pages which need to point to themselves. The __FILE__ constant contains the full path and filename of the current (i.e. included) file. )

    

* **REQUEST_URI** - (REQUEST_URI. The URI which was given in order to access this page; for instance, '/index.html'.)

    

* **PHP_AUTH_DIGEST** - (PHP_AUTH_DIGEST. When doing Digest HTTP authentication, this variable is set to the 'Authorization' header sent by the client (which you should then use to make the appropriate validation). )

    

* **PHP_AUTH_USER** - (PHP_AUTH_USER. When doing HTTP authentication, this variable is set to the username provided by the user.)

    

* **PHP_AUTH_PW** - (PHP_AUTH_PW. When doing HTTP authentication, this variable is set to the password provided by the user. )

    

* **AUTH_TYPE** - (AUTH_TYPE. When doing HTTP authentication, this variable is set to the authentication type. )

    

* **PATH_INFO** - (PATH_INFO. Contains any client-provided pathname information trailing the actual script filename but preceding the query string, if available. For instance, if the current script was accessed via the URL https://www.example.com/php/path_info.php/some/stuff?foo=bar, then $_SERVER['PATH_INFO'] would contain /some/stuff. )

    

* **ORIG_PATH_INFO** - (ORIG_PATH_INFO. Original version of 'PATH_INFO' before processed by PHP.)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Signature

A signature pad that allows users to draw smooth signatures on canvas element and export the signatures to PNG/JPG/SVG/WebP images.

**Field Type Parameters:**

1. Width - Width in pixels

2. Height - Height in pixels

3. Folder - Folder where all the signature image files will be saved. By default /images

4. Image Format

    

* **png** - (Save as PNG file)

    

* **webp** - (Save as WebP file)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

# Table Join

Left Join to another table. Finds the record to read a value of the specified field.

To read any field of joined table use {{ fieldname.get('joined_table_field_name') }}.

To read unprocessed value of the joined table field use this tag {{ fieldname.getvalue('joined_table_field_name') }}.

To read joined table record processed using the Layout use this tag {{ fieldname.layout('layout_name') }}.

**Field Type Parameters:**

1. Table - Type the name of the Custom Table to join/link with.
![Table](https://joomlaboat.com/images/components/ct/field-types/tablejoin-selecttable.png)


2. Field Name - Type the field name to return the value by default.

3. Filter - Limits records using filter. Example: [age>30] where age is a field in joining table

4. Dynamic Filter - Requires that joining table contains Table Join field to use it as a Category Filter

5. Order By - Field name to sort record by

6. Allow Unpublished - Allow to select unpublished records.

    

* (No - default)

    

* **true** - (Yes)

7. Selector - How to select records.

    

* **dropdown** - (Dropdown List)

    

* **radio** - (Radio)

8. Add Foreign Key - A FOREIGN KEY is a key used to link two tables together. The FOREIGN KEY constraint is used to prevent actions that would destroy links between tables. Like deleting table records. Enabled by default.

    

* **addforignkey** - (Add Foreign Key)

    

* **noforignkey** - (No Foreign Key)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s) to style the select box.

2. Attributes - Optional attributes like disabled, onclick, onchange, etc. that can be added to the select box.

3. Parent Selector - Used for multilevel dependencies like Country, City, Street.It comprises a list of parameters to set the sub-level select box. Examples include table names, field names, filters, and sorting orders for sub-level selections. It can accept multiple selectors for various dependencies.
`[city_table_name, city_field_name, allow_unpublished, filter, order_by]`It is also possible to pass multiple selectors.
`[ [city_table_name, city_field_name],[country_table_name, country_field_name, false]]`

4. Title Layout - Optional parameter setting the option title format instead of using the field name. The layout may contain field names surrounded by curly brackets for customized title display. Example: {{ name }} - {{ title }}
![Title Layout](https://joomlaboat.com/images/components/ct/field-types/tablejoin-customtitle.png)


5. Menu Item alias - Menu Item alias to open the link using specific menu item parameters.

6. Improve Select Box - An option to improve the select box UI, allowing in-list search and other visual enhancements.

    

* (Default. Standard)

    

* **improved** - (Improved (Joomla))

    

* **virtualselect** - (Improved (Virtual Select))

`{{ FieldName.edit("inputbox","data-importance='max'",[[city_table_name, city_field_name],[country_table_name, country_field_name, false]],"home-page","improved") }}`


**Read selected field processed value from joined table:**

1. Field name - Specify the name of the field to get the value from.

2. Value Processing Parameters - An array of parameters to process the Joined Table field value.

`{{ FieldName.get("birthdate",["Y-m-d"]) }}`


**Read unprocessed (pure) value of selected field of joined table:**

1. Field name - Specify the name of the field to get the value from.

`{{ FieldName.getvalue("country") }}`


**Render the value of joined table using selected layout.:**

1. Layout - The layout to render joined table record.

`{{ FieldName.layout("PersonProfileLayout") }}`

# Table Join List

Concatenated ID list referencing another table.

**Field Type Parameters:**

1. Table - Type the name of the Custom Table to join/link with.

2. Field Name - Type the field name to return the value from (that can be overwritten in Layout). You may type layout name instead. options: [layout:the layout or tablelesslayout:the layout]

3. Selector - How to select records.

    

* **single** - (Single Record (Dropdown List))

    

* **multi** - (Multiple Records (Dropdown List))

    

* **radio** - (Single Record (Radio))

    

* **checkbox** - (Multiple Records (Checkboxes))

    

* **multibox** - (Multiple Records (Add Record List, requires Dynamic Filter))

4. Filter - Limits records using filter. Example: [age>30] where age is a field in joining table

5. Dynamic Filter - Used with multibox, requires that joining table contain SQL Join field to use it as Category Filter

6. Order By - Field name to sort record by

7. Allow Unpublished - Allow to select unpublished records.

    

* (No - default)

    

* **true** - (Yes)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Field Name - Type the field name to return the value from. You may type layout name instead. options: [layout:layoutname or tablelesslayout:layoutname]

2. Order By - Field name to sort records by

3. List Separator - Separate record values with a character. i.e. ',' or ';'

`{{ FieldName("products","price",",") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

3. Parent Selector - Used for multilevel dependencies like Country, City, Street. List of parameters to set the sub-level select box. Example: [city_table_name, city_field_name, allow_unpublished(true or false), filter(population>100000), order_by(name)]. It is also possible to pass multiple selectors. Example: [ [city_table_name, city_field_name],[country_table_name, country_field_name, false]]

4. Title Layout - Optional parameter set the option title format, instead of the field name set in the field type parameter. The layout may contain the field names surrounded by curly brackets. Example: {{ name }} - {{ title }}
![Title Layout](https://joomlaboat.com/images/components/ct/field-types/tablejoin-customtitle.png)


`{{ FieldName.edit("","",[[city_table_name, city_field_name],[country_table_name, country_field_name, false]]) }}`


**Read processed value of selected field of joined table:**

1. Field name - Specify the name of the field to get the value from.

2. Show Published Records - Filter the list of records, show published, unpublished or all records.

    

* **published** - (Only Published)

    

* **unpublished** - (Only Unpublished)

    

* **all** - (Show all records)

3. List Separator - Separate record values with a character. i.e. ',' or ';'

`{{ FieldName.get("workers","any",",") }}`


**Read unprocessed value of selected field of joined table:**

1. Field name - Specify the name of the field to get the value from.

2. Show Published Records - Filter the list of records, show published, unpublished or all records.

    

* **published** - (Only Published)

    

* **unpublished** - (Only Unpublished)

    

* **all** - (Show all records)

3. List Separator - Separate record values with a character. i.e. ',' or ';'

`{{ FieldName.getvalue("workers","any",",") }}`


**Render the value of joined table using selected layout:**

1. Layout - The layout to render joined table record.

2. Show Published Records - Show published, unpublished or all records.

    

* **published** - (Only Published)

    

* **unpublished** - (Only Unpublished)

    

* **all** - (Show all records)

3. List Separator - Separate record values with a character. i.e. ',' or ';'

`{{ FieldName.layout("PeopleLayout","published",",") }}`

# Text Area (Multiple Lines)

Provides a textarea box for data entry.

**Field Type Parameters:**

1. Text Editor

    

* (Default. No Editor)

    

* **rich** - (Rich Text Editor)

2. Spell Checker

    

* (No - default)

    

* **spellcheck** - (Yes)

3. Maximum length - Maximum length of value to be held. The effective maximum length is less. The Joomla default utf8mb4 Character Set (4-Byte UTF-8 Unicode Encoding).

    

* **tiny** - (Up to 255 Bytes - 63 characters (TINYTEXT). TINYTEXT can store up to 63 Unicode characters i.e 255 bytes. It is suitable for storing short comments, captions, titles etc.)

    

* (Default. Up to 64KB - 16,383 characters (TEXT). TEXT can store up to 16,383 Unicode characters i.e 64KB. It is suitable for storing summaries of articles, short notices, captions, etc.)

    

* **medium** - (Up to 16MB - 4.2 million characters (MEDIUMTEXT). MEDIUMTEXT can store up to 4,194,303 Unicode characters i.e 16MB. It is suitable for larger text strings like books, research papers, and code backup.)

    

* **long** - (Up to 4GB - 10.7 billion characters (LONGTEXT). LONGTEXT can store up to 1,073,741,822 Unicode characters i.e 4GB. This is more than enough storage for any long-form text strings. For example, a book that MEDIUMTEXT can’t hold can be stored using LONGTEXT.)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Limit Output - Limits returning output text, first number of characters or first number of words.

    

* (Default. Returns as it is)

    

* **chars** - (Returns first number of characters from text that was trimmed, sanitized and HTML tags stripped.)

    

* **words** - (Returns first number of words from text that was trimmed, sanitized and HTML tags stripped.)

    

* **firstimage** - (Returns the link to a first image found in the text.)

2. Length - Number of characters or words.

    

* **Default** -

    

* **Min** - 1

    

* **Max** - 1024

3. Clean Braces - Deletes all braces from the text. (i.e. to avoid Content Plugins running.)

    

* (No - default)

    

* **true** - (Yes)

4. Clean Quotes - Deletes all quotes from the text.

    

* (No - default)

    

* **true** - (Yes)

`{{ FieldName(50,1,true,true) }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

3. Editor Width - Editor Width in Pixels or %

4. Editor Height - Editor Width in Pixels or %

`{{ FieldName.edit(100%,300px) }}`
# Text String (Single Line)

Provides a text box for data entry.

**Field Type Parameters:**

1. Length - Number of characters. Number of characters not bytes.

    

* **Default** - 255

    

* **Min** - 1

    

* **Max** - 1024

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Limit Output - Limits returning output text, first number of characters or first number of words.

    

* (Default. Returns as it is)

    

* **chars** - (Returns first number of characters from text that was trimmed, sanitized and HTML tags stripped.)

    

* **words** - (Returns first number of words from text that was trimmed, sanitized and HTML tags stripped.)

    

* **firstimage** - (Returns the link to a first image found in the text.)

2. Length - Number of characters or words.

    

* **Default** -

    

* **Min** - 1

    

* **Max** - 1024

3. Clean Braces - Deletes all braces from the text. (i.e. to avoid Content Plugins running.)

    

* (No - default)

    

* **true** - (Yes)

4. Clean Quotes - Deletes all quotes from the text.

    

* (No - default)

    

* **true** - (Yes)

`{{ FieldName("chars",1,true,true) }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

3. Autocomplete - Generate autocomplete data list using table records

    

* (No - default)

    

* **autocomplete** - (Yes)

`{{ FieldName.edit("my_CSS_class",onclick='alert(1);',"autocomplete") }}`
# Time

Time Selection. Saves value in ticks with defined offset. It will save time as number of seconds from the midnight by default.

**Field Type Parameters:**

1. Min (From) Time - Minimum (From) Time that can be selected.

2. Max (To) Time - Maximum (To) Time that can be selected.

3. Step - Selection increment. you can use h - hours, m - minutes, s - seconds. Or combined: 1h5m

4. Save Ticks - Save the value as a number of ticks.

5. Save Tick Offset - Save value offset number.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Format - PHP date format i.e. 'h:i:s'

`{{ FieldName("h:i") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

3. Format - Date/Time format. Similar to PHP date format.

# Translation

Use field name to add translations of some terms in your layout. This doesn't store anything in the Data Base.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

# URL / Link

URL / Link to some website. Must start with http:// or https://

**Field Type Parameters:**

1. CSS Class - Name or names of CSS class(s).

2. HTTPS Only - Allow only secure URL links

    

* (No - default)

    

* **true** - (Yes)

3. Domain Name - Must include following domain name, separate domain names with the comma. Example: youtube.com,youtu.be

4. Allow Query Parameters - Allow the URL links include query parameters like ?param1=apple

    

* (Yes - default)

    

* **false** - (No)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

# User

Provides User Selection box. It can also create a New User account if all necessary fields are set: Name,Email,User Group(s)

**Field Type Parameters:**

1. User Groups - Limit selection to certain User Group(s). Or used as default User Group(s) when creating a new user account.

2. Name - Used to create new user account only. Field tags, example: {{ firstname }} {{ lastname }}

3. Email - Used to create new user account only. Field tags, example: {{ email }}

4. Name Filter - Limit selection to certain name - name must content this characters.

5. Unique Users - Do not let assign same user account to different records.

    

* (Not Unique - default)

    

* **unique** - (Unique)

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# User Group

User Group Select Box.

**Field Type Parameters:**

1. Available User Groups - List of available User Groups separated by comma. User groups that can be selected. Super Users disabled by default. If the field is empty then all user groups will be visible to select except Super Users group.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# User Groups

User Group Multi Select Box

**Field Type Parameters:**

1. Selector - How to select records.

    

* **single** - (Single User Group (Select Box))

    

* **multi** - (Multiple User Groups (List Box))

    

* **radio** - (Single User Group (Radio))

    

* **checkbox** - (Multiple User Groups (Checkboxes))

2. Available User Groups - List of available User Groups separated by comma. User groups that can be selected. Super Users disabled by default. If the field is empty then all user groups will be visible to select except Super Users group.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# View Count

Stores the view count number.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

# View Time

Stores the date/time when record was last viewed.

**Value Processing Parameters:**

Example:`{{ FieldName }}`

1. Format - PHP date format i.e. 'Y-m-d' or Joomla defined format (language supported) if left empty. 'timestamp' to return the datetime in UNIX timestamp format.

`{{ FieldName("Y-m-d") }}`
**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

1. CSS Class - Name or names of CSS class(s).

2. Attributes - Optional attributes like: disabled, onclick, onchange etc.

# Virtual Field

Virtual field is the field the value of which is computed - processed by Twig. Similar to Layout processing.

**Field Type Parameters:**

1. Layout - Value Layout

2. Value Storage - Store the value for future use in search or order by.

    

* **virtual** - (Virtual (Computed on request))

    

* **storedstring** - (Stored String)

    

* **storedintegersigned** - (Stored Integer Signed)

    

* **storedintegerunsigned** - (Stored Integer Unsigned)

3. Length - Number of characters. Applied for Stored String only.

    

* **Default** - 255

    

* **Min** - 1

    

* **Max** - 4096

**Value Processing Parameters:**

Example:`{{ FieldName }}`

**Pure Value (Unprocessed):**

Example:

`{{ FieldName.value }}`

**Edit Record Parameters:**

Example:`{{ FieldName.edit }}`

